Option Strict On

Imports System.IO
Imports System.IO.Compression

Public Class Form1

    Private Enum enuAlgorithm
        Deflate = 0
        GZip = 1
    End Enum

    Private Sub btnBrowseCompress_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnBrowseCompress.Click
        'allows the user to browse for a file
        Dim objDLG As New OpenFileDialog
        With objDLG
            .InitialDirectory = "c:\"
            .Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*"
            .FilterIndex = 2
            .Multiselect = False
            .CheckFileExists = True
            If .ShowDialog = Windows.Forms.DialogResult.OK Then
                txtFileToCompress.Text = .FileName
            End If
            .Dispose()
        End With
        objDLG = Nothing
    End Sub

    Private Sub btnCompress_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCompress.Click
        'Some validation on the file name
        If Not ValidateFile(txtFileToCompress.Text.Trim) Then
            txtFileToCompress.Focus()
            Exit Sub
        End If

        'Identify the algorithm to use
        Dim intAlgorithm As enuAlgorithm
        If optAlgorithmDeflate.Checked Then
            intAlgorithm = enuAlgorithm.Deflate
        Else
            intAlgorithm = enuAlgorithm.GZip
        End If

        'Call the compression method
        CompressFile(txtFileToCompress.Text, intAlgorithm)
    End Sub

    Private Sub btnUnCompress_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnUnCompress.Click
        'Some validation on the file name
        If Not ValidateFile(txtFileToCompress.Text.Trim) Then
            txtFileToCompress.Focus()
            Exit Sub
        End If

        'Identify the algorithm to use
        Dim intAlgorithm As enuAlgorithm
        If optAlgorithmDeflate.Checked Then
            intAlgorithm = enuAlgorithm.Deflate
        Else
            intAlgorithm = enuAlgorithm.GZip
        End If

        'Call the compression method
        UnCompressFile(txtFileToCompress.Text, intAlgorithm)
    End Sub

    Private Sub CompressFile(ByVal pFileName As String, ByVal pAlgorithm As enuAlgorithm)
        Dim stmInFile, stmOutFile As FileStream
        Dim strExtension As String
        If pAlgorithm = enuAlgorithm.Deflate Then
            strExtension = ".compress"
        Else
            strExtension = ".gz"
        End If

        ' Open the input file as a FileStream object.
        stmInFile = New FileStream(pFileName, FileMode.Open)

        ' Open the output file as a FileStream object.
        stmOutFile = New FileStream(pFileName + strExtension, FileMode.Create)

        ' Create a new stream for the compressed data.
        Dim stmCompressed As Stream
        If pAlgorithm = enuAlgorithm.Deflate Then
            stmCompressed = New DeflateStream(stmOutFile, CompressionMode.Compress)
        Else
            stmCompressed = New GZipStream(stmOutFile, CompressionMode.Compress)
        End If

        Dim arrBuffer(4095) As Byte
        Do
            'Read a chunk of the input file
            Dim intReadBytes As Integer = stmInFile.Read(arrBuffer, 0, arrBuffer.Length)
            'Detect the end of file
            If intReadBytes = 0 Then Exit Do
            'Write the compressed chunk
            stmCompressed.Write(arrBuffer, 0, intReadBytes)
        Loop

        'Close the streams
        stmCompressed.Close()
        stmInFile.Close()
        stmOutFile.Close()

        'Shows the result of the compression
        MessageBox.Show("The original (uncompressed) file contains " + New FileInfo(pFileName).Length.ToString + " bytes." + _
                        Environment.NewLine + _
                        "The compressed file contains " + New FileInfo(pFileName + strExtension).Length.ToString + " bytes.", _
                        "Compression completed", MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub

    Private Sub UnCompressFile(ByVal pFileName As String, ByVal pAlgorithm As enuAlgorithm)
        Dim stmInFile, stmOutFile As FileStream
        Dim strExtension As String

        'You will need to find the original file name and remove the previous extension!
        If pAlgorithm = enuAlgorithm.Deflate Then
            strExtension = ".txt"
        Else
            strExtension = ".txt"
        End If

        ' Open the input file as a FileStream object.
        stmInFile = New FileStream(pFileName, FileMode.Open)

        ' Open the output file as a FileStream object.
        stmOutFile = New FileStream(pFileName + strExtension, FileMode.Create)

        ' Create a new stream for the uncompressed data.
        Dim stmCompressed As Stream
        If pAlgorithm = enuAlgorithm.Deflate Then
            stmCompressed = New DeflateStream(stmInFile, CompressionMode.Decompress)
        Else
            stmCompressed = New GZipStream(stmInFile, CompressionMode.Decompress)
        End If

        Dim arrBuffer(4095) As Byte
        Do
            'Read a chunk of the input file
            Dim intReadBytes As Integer = stmCompressed.Read(arrBuffer, 0, arrBuffer.Length)
            'Detect the end of file
            If intReadBytes = 0 Then Exit Do
            'Write the compressed chunk
            stmOutFile.Write(arrBuffer, 0, intReadBytes)
        Loop

        'Close the streams
        stmCompressed.Close()
        stmInFile.Close()
        stmOutFile.Close()

        'Shows the result of the compression
        MessageBox.Show("The original (compressed) file contains " + New FileInfo(pFileName).Length.ToString + " bytes." + _
                        Environment.NewLine + _
                        "The uncompressed file contains " + New FileInfo(pFileName + strExtension).Length.ToString + " bytes.", _
                        "Uncompression completed", MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub

    Private Function ValidateFile(ByVal pFileName As String) As Boolean
        'Some validation on the file name
        If pFileName.Length = 0 Then
            MessageBox.Show("You must provide a file name!", "Information required", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return False
        End If

        If Not IO.File.Exists(pFileName) Then
            MessageBox.Show("You must provide an existing file name!", "Information required", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return False
        End If

        Return True
    End Function
End Class